import os

BC_FROM_BRANCH = os.getenv('BC_FROM_BRANCH', "")
BC_TO_BRANCH = os.getenv('BC_TO_BRANCH', "")
BC_PR_ID = os.getenv('BC_PR_ID', "")
BC_PR_URL = os.getenv('BC_PR_URL', "")
BC_COMMIT_HASH = os.getenv('BC_COMMIT_HASH', "")
BC_COMMIT_URL = os.getenv('BC_COMMIT_URL', "")
BC_AUTHOR = os.getenv('BC_AUTHOR', "")
BC_AUTHOR_URL = os.getenv('BC_AUTHOR', "")
BC_RUN_ID = os.getenv('BC_RUN_ID', "")
BC_RUN_URL = os.getenv('BC_RUN_URL', "")
BC_REPOSITORY_URL = os.getenv('BC_REPOSITORY_URL', "")



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck


class DocDBLogging(BaseResourceCheck):
    def __init__(self):
        name = "Ensure DocDB Logging is enabled"
        id = "CKV_AWS_85"
        supported_resources = ['AWS::DocDB::DBCluster']
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        log_types = ["profiler", "audit"]
        if 'Properties' in conf.keys():
            if 'EnableCloudwatchLogsExports' in conf['Properties'].keys():
                if all(elem in conf['Properties']['EnableCloudwatchLogsExports'] for elem in log_types):
                    return CheckResult.PASSED
        return CheckResult.FAILED


check = DocDBLogging()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck

class CodeBuildProjectEncryption(BaseResourceCheck):
    def __init__(self):
        name = "Ensure that CodeBuild Project encryption is not disabled"
        id = "CKV_AWS_78"
        supported_resources = ['AWS::CodeBuild::Project']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        # Only Fail if Artifact Type is S3 and EncryptionDisabled is True.
        artifact_type = ""
        encryption_disabled = False
        if 'Properties' in conf.keys():
            if 'Artifacts' in conf['Properties'].keys():
                if 'Type' in conf['Properties']['Artifacts'].keys():
                    artifact_type = conf['Properties']['Artifacts']['Type']
                if 'EncryptionDisabled' in conf['Properties']['Artifacts'].keys(): 
                    encryption_disabled = conf['Properties']['Artifacts']['EncryptionDisabled']
                if artifact_type == "S3" and encryption_disabled is True:
                    return CheckResult.FAILED
        return CheckResult.PASSED


check = CodeBuildProjectEncryption()



import logging
import re
import shutil

TAG_PATTERN = re.compile(r'\?(ref=)(?P<tag>v\d+.\d+.\d+)')
try:
    from git import Repo
    git_import_error = None
except ImportError as e:
    git_import_error = e


from checkov.common.goget.base_getter import BaseGetter


class GitGetter(BaseGetter):
    def __init__(self, url, create_clone_and_result_dirs=True):
        self.logger = logging.getLogger(__name__)
        self.create_clone_and_res_dirs = create_clone_and_result_dirs
        self.tag = ''

        search_tag = re.search(TAG_PATTERN, url)
        if search_tag:
            self.tag = search_tag.groupdict().get('tag')
        url = re.sub(TAG_PATTERN, '', url)

        super().__init__(url)

    def do_get(self):
        if git_import_error is not None:
            raise ImportError("Unable to load git module (is the git executable available?)") \
                from git_import_error

        clone_dir = self.temp_dir + "/clone/" if self.create_clone_and_res_dirs else self.temp_dir
        result_dir = self.temp_dir + "/result/"

        if ".git//" in self.url:
            git_url, internal_dir = self.url.split(".git//")
            self._clone(git_url + ".git", clone_dir, result_dir, internal_dir)
        else:
            self._clone(self.url, clone_dir, result_dir)

        return result_dir

    def _clone(self, git_url, clone_dir, result_dir, internal_dir=''):
        self.logger.debug("cloning {} to {}".format(self.url, clone_dir))
        if self.tag:
            Repo.clone_from(git_url, clone_dir, b=self.tag)
        else:
            Repo.clone_from(git_url, clone_dir)
        if self.create_clone_and_res_dirs:
            shutil.copytree(clone_dir + internal_dir, result_dir)



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.arm.base_resource_check import BaseResourceCheck

class AzureManagedDiscEncryption(BaseResourceCheck):
    def __init__(self):
        name = "Ensure Azure managed disk have encryption enabled"
        id = "CKV_AZURE_2"
        supported_resources = ['Microsoft.Compute/disks']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if "properties" in conf:
            if "encryptionSettings" in conf["properties"]:
                if "enabled" in conf["properties"]["encryptionSettings"]:
                    if str(conf["properties"]["encryptionSettings"]["enabled"]).lower() == "true":
                        return CheckResult.PASSED
        return CheckResult.FAILED

check = AzureManagedDiscEncryption()



from checkov.cloudformation.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck
from checkov.common.models.enums import CheckCategories


class S3PublicACLWrite(BaseResourceNegativeValueCheck):
    def __init__(self):
        name = "Ensure the S3 bucket does not allow WRITE permissions to everyone"
        id = "CKV_AWS_57"
        supported_resources = ['AWS::S3::Bucket']
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_forbidden_values(self):
        return  ['PublicReadWrite']

    def get_inspected_key(self):
        return 'Properties/AccessControl'

check = S3PublicACLWrite()



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class NeptuneClusterLogging(BaseResourceCheck):
    def __init__(self):
        name = "Ensure Neptune logging is enabled"
        id = "CKV_AWS_101"
        supported_resources = ['aws_neptune_cluster']
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        log_types = ["audit"]
        if 'enable_cloudwatch_logs_exports' in conf.keys():
            if all(elem in conf['enable_cloudwatch_logs_exports'] for elem in log_types):
                return CheckResult.PASSED
        return CheckResult.FAILED

check = NeptuneClusterLogging()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck


class IAMPolicyAttachedToGroupOrRoles(BaseResourceCheck):
    def __init__(self):
        name = "Ensure IAM policies are attached only to groups or roles (Reducing access management complexity may " \
               "in-turn reduce opportunity for a principal to inadvertently receive or retain excessive privileges.)"
        id = "CKV_AWS_40"
        supported_resources = ['AWS::IAM::Policy']
        categories = [CheckCategories.IAM]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        """
        Looks for users attached to an IAM policy
        https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-policy.html
        :param conf: aws_iam_policy
        :return: <CheckResult>
        """
        if 'Properties' in conf.keys():
            props_conf = conf['Properties']
            if 'Users' in props_conf.keys():
                return CheckResult.FAILED
            return CheckResult.PASSED
        return CheckResult.FAILED


check = IAMPolicyAttachedToGroupOrRoles()
